interface Predicate<T> {
	public boolean call(T obj);
}

interface Projection<T, RT> {
	public RT call(T obj);
}

class QueryArrayList<T> extends java.util.ArrayList<T> {

	QueryArrayList<T> where(Predicate<T> pre) {
		QueryArrayList<T> newList = new QueryArrayList<T>();
		for (T obj : this) {
			if (pre.call(obj)) {
				newList.add(obj);
			}
		}
		return newList;
	}

	<R> QueryArrayList<R> project(Projection<T, R> sel) {
		QueryArrayList<R> newList = new QueryArrayList<R>();
		for (T obj : this) {
			newList.add(sel.call(obj));
		}
		return newList;
	}
}

class Person {
	public String name; // Nachname
	public String firstname; // Vorname
	public boolean isMale; // maennlich?

	public Person(String name, String firstname, boolean isMale) {
		this.name = name;
		this.firstname = firstname;
		this.isMale = isMale;
	}
}

class MaleChecker implements Predicate<Person> {

	@Override
	public boolean call(Person p) {
		return p.isMale;
	}

}

class NameSelector implements Projection<Person, String> {

	@Override
	public String call(Person p) {
		return p.name;
	}

}

public class UE23Aufgabe9 {

	public static void main(String[] args) {
		QueryArrayList<Person> personList = new QueryArrayList<Person>();
		System.out.println("Personenerzeugung:");
		char choice = IO.readChar("Weitere Person (j/n)?");
		while (choice == 'j') {
			String name = IO.readString("Name: ");
			String firstname = IO.readString("Vorname: ");
			boolean isMale = IO.readBoolean("maennlich (true/false)?");
			personList.add(new Person(name, firstname, isMale));
			choice = IO.readChar("Weitere Person (j/n)?");
		}

		QueryArrayList<Person> menList = personList.where(new MaleChecker());
		QueryArrayList<String> nameList = menList.project(new NameSelector());

		System.out.println("\nNachnamen:");
		for (String name : nameList) {
			System.out.println(name);
		}
	}
}
